/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces a Spin Button.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class SpinButton extends Canvas {
	
	/** When SpinButton increase the value, parent will get 
	 *  the event with this id. 
	 * @see #SPIN_DOWN
	 */
    public static final int SPIN_UP   = 731021;

	/** When SpinButton decrease the value, parent will get 
	 *  the event with this id. 
	 * @see #SPIN_UP
	 */
    public static final int SPIN_DOWN = 731022;

	/** Current orientation of SpinButton.
	 *  True = LEFT-RIGHT, False = UP-DOWN
	 * @see #defaultOrientation
	 * @see #setOrientation
	 * @see #getOrientation
	 */
    protected boolean orientation;

	/** Default orientation of SpinButton. Currently false (UP-DOWN).
	 * @see #defaultOrientation
	 * @see #setOrientation
	 * @see #getOrientation
	 */
	public static final boolean defaultOrientation = false;

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
	public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Current width of the 3D border around the component
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
	protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    public static final int defaultBorderWidth = 1;

	/** Minimum of the range of the SpinButton.
	 * @see #defaultMinimum
	 * @see #setMinimum
	 * @see #getMinimum
	 */
    protected int minimum;

	/** Maximum of the range of the SpinButton.
	 * @see #defaultMaximum
	 * @see #setMaximum
	 * @see #getMaximum
	 */
    protected int maximum;
	
	/** Step of the SpinButton.
	 * @see #defaultStep
	 * @see #setStep
	 * @see #getStep
	 */
	protected int step;

	/** Value of the SpinButton. Default 0.
	 * @see #setValue
	 * @see #getValue
	 */
	protected int value;

	/** Default minimum. Currently 0.
	 * @see #minimum
	 * @see #setMinimum
	 * @see #getMinimum
	 */
	public static final int defaultMinimum = 0;

	/** Default maximum. Currently 100.
	 * @see #maximum
	 * @see #setMaximum
	 * @see #getMaximum
	 */
	public static final int defaultMaximum = 100;

	/** Default step. Currently 1.
	 * @see #step
	 * @see #setStep
	 * @see #getStep
	 */
	public static final int defaultStep = 1;

	/** Adjusts the amount of time that elapses before a increment
	 *  (or decrement) begins repeating when you hold down a mouse
	 *  button. [ms]
	 * @see #setDelay
	 * @see #getDelay
	 * @see #defaultRepeatDelay
	 */
	protected int repeatDelay;

	/** Adjusts the speed at which a increment (or decrement) 
	 *  repeats when you hold down a mouse button. [ms]
	 * @see #setRate
	 * @see #getRate
	 * @see #defaultRepeatRate
	 */
	protected int repeatRate;
	
	/** Default value of repeatDelay. Currently 300 ms.
	 * @see #setDelay
	 * @see #getDelay
	 * @see #repeatDelay
	 */
	public static final int defaultRepeatDelay = 300;

	/** Default value of repeatRate. Currently 70 ms.
	 * @see #setRate
	 * @see #getRate
	 * @see #repeatRate
	 */
	public static final int defaultRepeatRate = 70;

	/** Spin repeat thread. When the SpinButton is holded this thread
	 *  runs and regulary sends the events to SpinButton.
	 */
	protected SpinRepeatThread rt = null;

	/** Flag if the SpinRepeatThread is currently running. */
	protected boolean running = false;

	/** Current direction of the run of the SpinRepeatThread. */
	protected int runningDir = 0;

	/** If propagate is set to true all events are propagate and don't
	 *  catch. This flag is used for xawt component during design time.
	 */
	public boolean propagate = false;

	/** Create new SpinButton with all default values. */
	public SpinButton() {
		rt = null;
		setDefaults();
	}

	private void setDefaults() {
		orientation = defaultOrientation;
    	marginColor = defaultBorderColor;
		margin = defaultBorderWidth;
		minimum = defaultMinimum;
		maximum = defaultMaximum;
		step = defaultStep;
		value = 0;
		repeatDelay = defaultRepeatDelay;
		repeatRate = defaultRepeatRate;
	}

	/** Sets the new orientation.
	 * @param aDir new value of orientation.
	 * @see #orientation
	 * @see #defaultOrientation
	 * @see #getOrientation
	 */
	public void setOrientation(boolean aDir) {
		orientation = aDir;
		switchStop();
		repaint();
	}

	/** Gets the current orientation of SpinButton.
	 * @return value of orientation.
	 * @see #orientation
	 * @see #defaultOrientation
	 * @see #setOrientation
	 */
	public boolean getOrientation() {
		return orientation;
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
		switchStop();
        repaint();
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets a minimum of the range of the SpinButton. If value
	 *  or maximum fall out of acceptable values they are adjusted.
	 * @param aMin New minimum.
	 * @see #getMinimum
	 */
	public void setMinimum(int aMin) {
		minimum = aMin;
		if (value < minimum)
			value = minimum;
		if (maximum < minimum)
			maximum = minimum;
		switchStop();
        repaint();
	}

	/** Gets the current minimum of the range of SpinButton.
	 * @return Minimum.
	 * @see #setMinimum
	 */
	public int getMinimum() {
		return minimum;
	}
	
	/** Sets a maximum of the range of the SpinButton. If value
	 *  or minimum fall out of acceptable values they are adjusted.
	 * @param aMax New maximum.
	 * @see #getMinimum
	 */
	public void setMaximum(int aMax) {
		maximum = aMax;
		if (value > maximum)
			value = maximum;
		if (maximum < minimum)
			minimum = maximum;
		switchStop();
        repaint();
	}

	/** Gets the current maximum of the range of SpinButton.
	 * @return Maximum.
	 * @see #setMaximum
	 */
	public int getMaximum() {
		return maximum;
	}

	/** Sets a new value of the SpinButton. If value is outside
	 *  the ranges it is set to nearest acceptable value.
	 * @param aValue New value.
	 * @see #getValue
	 */
	public void setValue(int aValue) {
		value = aValue;
		if (value < minimum)
			value = minimum;
		if (value > maximum)
			value = maximum;
		switchStop();
        repaint();
	}

	/** Gets the current value of the SpinButton.
	 * @return Value.
	 * @see #setValue
	 */
	public int getValue() {
		return value;
	}

	/** Sets a new step of the SpinButton.
	 * @param aStep New step.
	 * @see #getStep
	 */
	public void setStep(int aStep) {
		step = aStep;
		switchStop();
        repaint();
	}

	/** Gets the current step of the SpinButton.
	 * @return Step.
	 * @see #setStep
	 */
	public int getStep() {
		return step;
	}

	/** Sets new value of repeatDelay variable.
	 * @param aDelay New delay.
	 * @see #repeatDelay
	 * @see #getDelay
	 */
	public void setDelay(int aDelay) {
		repeatDelay = aDelay;
		switchStop();
        repaint();
	}

	/** Gets the current value of repeatDelay variable.
	 * @return Delay.
	 * @see #repeatDelay
	 * @see #setDelay
	 */
	public int getDelay() {
		return repeatDelay;
	}

	/** Sets new value of repeatRate variable.
	 * @param aRate New rate.
	 * @see #repeatRate
	 * @see #getRate
	 */
	public void setRate(int aRate) {
		repeatRate = aRate;
		switchStop();
        repaint();
	}

	/** Gets the current value of rate variable.
	 * @return Rate.
	 * @see #repeatRate
	 * @see #setRate
	 */
	public int getRate() {
		return repeatRate;
	}

	public void paint(Graphics g) {
		Dimension d = size();

		int left = 0;
		int top = 0;
		int w = d.width - 1;
		int h = d.height - 1;

		g.setColor(getBackground());
		g.fillRect(left,top,w,h);
		if (orientation) {
			w = w / 2;
			paintBorder(g, left, top, w, h, running && (runningDir == SPIN_DOWN), SPIN_DOWN);
			left += w + 1;
			w = d.width - 1 - left;
			paintBorder(g, left, top, w, h, running && (runningDir == SPIN_UP), SPIN_UP);
		}
		else {
			h = h / 2;
			paintBorder(g, left, top, w, h, running && (runningDir == SPIN_UP), SPIN_UP);
			top += h + 1;
			h = d.height - 1 - top;
			paintBorder(g, left, top, w, h, running && (runningDir == SPIN_DOWN), SPIN_DOWN);
		}
	}

	private void paintBorder(Graphics g, int x, int y, int w, int h, boolean isDown, int aDir) {
	    g.setColor(Color.black);
		if (!isDown) {
		    g.drawLine(x, y + h, x + w, y + h);
		    g.drawLine(x + w, y, x + w, y + h);
		}
		else {
		    g.drawLine(x, y, x + w, y);
		    g.drawLine(x, y, x, y + h);
		    x++;
		    y++;
		}
		w--;
		h--;
		g.setColor(marginColor);
		for (int i = 0; i < margin; i++) {
			g.draw3DRect(x, y, w, h, !isDown);
			x++;
			y++;
			w -= 2;
			h -= 2;
		}
		paintArrow(g, x, y, w, h, aDir);
	}

	private void paintArrow(Graphics g, int x, int y, int w, int h, int aDir) {
		if ((w <= 0) || (h <= 0))
			return;
		int wd = w / 4;
		int hd = h / 4;
        int[] xP = new int[3];
        int[] yP = new int[3];
		
		if (orientation) {
			if (aDir == SPIN_UP) {
				xP[0] = x + wd;
				xP[2] = x + w - wd;
			}
			else {
				xP[0] = x + w - wd;
				xP[2] = x + wd;
			}
			xP[1] = xP[0];
			yP[0] = y + hd;
			yP[1] = y + h - hd;
			yP[2] = y + h/2;
		}
		else {
			if (aDir == SPIN_UP) {
				yP[0] = y + h - hd;
				yP[2] = y + hd;
			}
			else {
				yP[0] = y + hd;
				yP[2] = y + h - hd;
			}
			yP[1] = yP[0];
			xP[0] = x + wd;
			xP[1] = x + w - wd;
			xP[2] = x + w/2;
		}
		if (((aDir == SPIN_UP)&&(value >= maximum))||((aDir == SPIN_DOWN)&&(value <= minimum))) {
			Color fg = getForeground();
			Color bg = getBackground();
			g.setColor(new Color((fg.getRed() + 2*bg.getRed())/3, (fg.getGreen() + 2*bg.getGreen())/3,
				(fg.getBlue() + 2*bg.getBlue())/3));
		}
		else 
			g.setColor(getForeground());
		g.fillPolygon(xP, yP, 3);
	}	

    public boolean mouseDown(Event event, int x, int y) {
		if (propagate)
			return false;
		Dimension d = size();
		int newDir = 0;
		if (orientation) {
			if (x <= ((d.width - 1) / 2))
				newDir = SPIN_DOWN;
			else
				newDir = SPIN_UP;
		}
		else {
			if (y <= ((d.height - 1) / 2))
				newDir = SPIN_UP;
			else
				newDir = SPIN_DOWN;
		}
		if (((newDir == SPIN_UP)&&(value >= maximum))||((newDir == SPIN_DOWN)&&(value <= minimum)))
			return true;
		switchRun(newDir);
		repaint();
		return true;
	}
	
    public boolean mouseUp(Event event, int x, int y) {
		if (propagate)
			return false;
		boolean r = running;
		switchStop();
		if (r)
			repaint();
		return true;
	}

	protected synchronized void switchRun(int aDirect) {
		if (running)
			rt.finish = true;
		rt = new SpinRepeatThread(this, repeatDelay, repeatRate, aDirect);
		rt.start();
		runningDir = aDirect;
		running = true;
	}
	
	protected synchronized void switchStop() {
		if (rt == null)
			return;
		rt.finish = true;
		running = false;
	}

    public boolean handleEvent(Event event) {
		if (propagate)
			return false;
    	if (event.id == SPIN_UP) {
			value += step;
			if (value >= maximum) {
				value = maximum;
				switchStop();
				repaint();
			}
   	    	return false;
    	}
		else if (event.id == SPIN_DOWN) {
			value -= step;
			if (value <= minimum) {
				value = minimum;
				switchStop();
				repaint();
			}
   	    	return false;
    	}
    	return super.handleEvent(event);
	}	

    public Dimension minimumSize() {
        return countSize();
    }

    public Dimension preferredSize() {
        return countSize();
    }

	private Dimension countSize() {
		int x = 13 + 2 * margin;
		int y = x;
		if (orientation)
			x = x + x;
		else
			y = y + y;
		return new Dimension(x, y);
	}

}

class SpinRepeatThread extends Thread {
	SpinButton parent;
	int delay;
	int rate;
	boolean finish;
	int direct;

	public SpinRepeatThread(SpinButton aParent, int aDelay, int aRate, int aDirect) {
		parent = aParent;
		delay = aDelay;
		rate = aRate;
		finish = false;
		direct = aDirect;
	}

	public void run() {
		Event repeatEvent = new Event(parent, direct, null); 
		parent.postEvent(repeatEvent);
		parent.repaint();
		try {
			sleep(delay);
		} catch (InterruptedException e) {}
		while (true) {
			if (finish) break;
			parent.postEvent(repeatEvent);
			if (finish) break;
			try {
				sleep(rate);
			} catch (InterruptedException e) {}
		}
	}
}
