/** Output window class
* @author: Filip Dvorak
* @version: 1.2
*/

package xelfi.compiler;

import java.awt.*;
import java.awt.List;
import java.util.*;

import xelfi.top.*;

  /** Output window. Anyone can write to it. Its behaviour is the same like System.out.print..
    functions - text is buffered until newline character comes. */
public class Output extends TopDialog
{
  private List l;
     // for each item in the List (except the last one) exist an item in the callback
  private Vector callback;
  private StringBuffer buf;  // buffer of the unterminated line

    // set if the text in the bufferred line of the List is not consistent
  private boolean bufferDirty;
  private static final int INIT_BUF_SIZE = 200;    // size of the StringBuffer

    /** Creates an empty Output window */
  public Output()
  {
    super("Output", false);
    reshape(400, 100, 400, 350);
    setLayout(new BorderLayout());
    l = new List();
    add("Center", l);
    callback = new Vector(64, 64);
    buf = new StringBuffer(INIT_BUF_SIZE);
    l.addItem("");    // bufferred line of text
    bufferDirty = false;
  }

    /** Last unterminated bufferred line is flushed out. No '\n' character is appended so
      that next print will continue on this flushed line. 
      @return an index of this line */
  public int flush()
  {
    int index;
    if(bufferDirty)
    {
      l.replaceItem(buf.toString(), index = l.countItems() - 1);
      l.makeVisible(index);
      bufferDirty = false;
      return index;
    }
    else return l.countItems() - 1;
  }

    /** Called when want to terminate the current line and begin a new one. Callback cb is set for
      an old line. 
      @return the index of the old line */
  protected int newLine(OutputCallback cb)
  {
    int i = flush();
    callback.addElement(cb);  // has the index i
    l.addItem("");
    buf = new StringBuffer(INIT_BUF_SIZE);
    bufferDirty = false;
    return i;
  }

    /** This function prints the String into Output and appends one new line character.
      The String must be only one line - it may not contain any '\n' characters.).
      @param text string to print
      @parab cb callback that will be invoked when user selecteds the text.
      @return the text's index which can be later passed to OutputCallback's methods. */
  public synchronized int outLine(String text, OutputCallback cb)
  {
    buf.append(text);
    bufferDirty = true;
    return newLine(cb);
  }

    /** This function prints the String into Output. Strings in output are
      buffered until newline char appears, or until explicit call of flush() method.
      @param text string to print. */
  public synchronized void print(String text)
  {
    String s1 = null, s2 = "";
    StringTokenizer t = new StringTokenizer(text, "\n", true);
    while(t.hasMoreTokens())
      if((s1 = t.nextToken()).equals("\n")) newLine(null);
      else
      {
        buf.append(s1);
        bufferDirty = true;
      }
  }

    /** This function prints the String into Output and appends one new line character.
      Strings in output are buffered until newline char appears, or until explicit call of
      flush() method.
      @param text string to print. */
  public synchronized void println(String text)
  {
    print(text);
    newLine(null);
  }

    /** Clears all text in the Output, invokes OutputCallback.clear() methods for each
      cleared string */
  public synchronized void clear()
  {
    Enumeration items = callback.elements();
    OutputCallback c = null;
    int i = 0;

    l.clear();
    while(items.hasMoreElements())
      if((c = (OutputCallback)items.nextElement()) != null) c.clear(i++);
      else i++;
    callback.removeAllElements();
    buf = new StringBuffer(INIT_BUF_SIZE);
    l.addItem("");    // bufferred line of text
    bufferDirty = false;
  }

    /** Makes text (printed by outLine method) visualy selected.
      @param callbackIndex index of a line of a text returned by outLine() method
    */
  public void select(int callbackIndex)
  {
    l.select(callbackIndex);
  }

  public boolean handleEvent(Event event)
  {
    switch(event.id)
    {
      case Event.WINDOW_DESTROY:
        hide();
        return true;
      case Event.LIST_SELECT:
        if(event.target == l)
        {
          listSelection((Integer)event.arg);
          return true;
        }
        else return false;
      case Event.ACTION_EVENT:
        if(event.target == l)
        {
          listAction();
          return true;
        }
        else return false;
    }
    return super.handleEvent(event);
  }

  private void listSelection(Integer i)
  {
      // index i from the List is can be used to access vector item
    OutputCallback handler = null;
    int index = i.intValue();
    if(l.countItems() - 1 > index &&   // not the last line
      (handler = (OutputCallback)callback.elementAt(index)) != null) handler.selection(index);
  }

  private void listAction()
  {
      // index i from the List is can be used to access vector item
    OutputCallback handler = null;
    int index = l.getSelectedIndex();
    if(l.countItems() - 1 > index &&
      (handler = (OutputCallback)callback.elementAt(index)) != null) handler.action(index);
  }
}





































