/**
* Xelfi Environment
*/

package xelfi.editor;

/** Provides operation on the TextEditorInfo interface. It done in
* this way, because it is not allowed to have methods (even static)
* in interfaces.
*
* @author Jaroslav Tulach
* @version 1.0
*/
public abstract class TextEditorOperation extends Object {

  /** Deletes the selected block in the editor. But only if it is allowed.
  * @param info info about editor
  * @exception NotAllowedException if the insert is not allowed
  */
  public static void clear (TextEditorInfo info) throws NotAllowedException {
    info.getSelectedBlock ().delete ();
  }

  /** Cuts selected block.
  * @param info editor info
  * @exception NotAllowedException if the insert is not allowed
  */
  public static void cut (TextEditorInfo info) throws NotAllowedException {
    copy (info);
    clear (info);
  }

  /** Copies block to clipboard.
  * @param info editor info
  */
  public static void copy (TextEditorInfo info) {
    Clipboard.put (info.getSelectedBlock ());
  }

  /** Pastes a block from clipboard.
  * @param info editor info
  * @exception NotAllowedException if the insert is not allowed
  */
  public static void paste (TextEditorInfo info) throws NotAllowedException {
    Position p = info.getCursorPos ();
    info.setSelectedBlock (Block.between (p.insert (Clipboard.get ()), p));
  }

  /** Moves one page down.
  * @param info info about editor
  */
  public static void pageDown (TextEditorInfo info) {
    int lines = info.numberOfVisibleLines ();
    info.setCursorPos (info.getCursorPos ().safeDown (lines));
  }

  /** Moves one page up.
  * @param info info about editor
  */
  public static void pageUp (TextEditorInfo info) {
    int lines = info.numberOfVisibleLines ();
    info.setCursorPos (info.getCursorPos ().safeUp (lines));
  }

  /** Places bookmark.
  * @param info the editor info
  * @param name name of the bookmark
  * @param pos the position to set bookmark to
  */
  public static void placeBookmark (
    TextEditorInfo info, String name, Position pos
  ) {
    info.getHashtable ().put (
      "xelfi.editor.TextEditorOperatin@placeBookmark:" + name, pos
    );
  }

  /** Places bookmark to the cursor position.
  * @param info the editor info
  * @param name name of the bookmark
  */
  public static void placeBookmark (
    TextEditorInfo info, String name
  ) {
   placeBookmark (info, name, info.getCursorPos ());
  }

  /** Gets bookmark.
  * @param info editor info
  * @param name bookmark name
  * @return the position of bookmark or null
  */
  public static Position getBookmark (TextEditorInfo info, String name) {
    try {
      return (Position)info.getHashtable ().get (
        "xelfi.editor.TextEditorOperatin@placeBookmark:" + name
      );
    } catch (Exception ex) {
      return null;
    }
  }
}




// Generated from substitution template 'Class' Blocks:Class:22,13,41,13; Package:8,4,20,4; 
