/**
* Xelfi programming environment
*/

package xelfi.templates;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import xelfi.awt.XFTree;
import xelfi.awt.XFTreeNode;

import xelfi.util.EnhProperties;
import xelfi.editor.XelfiFile;
import xelfi.editor.Source;
import xelfi.editor.Editor;
/**
* This is class in instantiated for every template in system.
* Serves especially as a handle to services provided by
* TemplateManager.
* @see TemplateManager
*
* @version $Id$
* @autor Jaroslav Tulach
*/

public abstract class Template extends Object {
  /** its template manager */
  TemplateManager manager;

  /** Super template. The ancessor. */
  Template supr;

  /** The name of template */
  String name;

  /** Indicates that this Template has sub templates.
  * This flag is set in Registration.register method to
  * the lastly inserted son.
  */
  XFTreeNode hasSubTemplates = null;

  /** Access to XFTreeNode that contains this template.
  */
  XFTreeNode treeNode;

  /** This flag is set to true, if this template should be stored to
  * properties. By default it is true, if loaded from Properties by
  * call to loadTemplates.
  */
  boolean saveToProperties = false;

  /** Basic template for all templates
  */
  static Template basic = new TemplateImpl ("Plain");

  /** Inicializes templates. Registers all
  * loaders and savers.
  */
  static {
    TemplateLoader tl = new TemplateLoader ();
    Editor.addLoader (tl);
  };
  
  /** Registers editor loader for finding templates.
  */
  public static void initialize () {
    // when a call to this method is done the previous static
    // code is executed
  };
  
  /** Forbids outclass construction of a template.
  * @param name the name of template
  * @param supr the super template
  * @param manager the manager for this template
  */
  Template (String name, Template supr, TemplateManager manager) {
    this.name = name;
    this.supr = supr;
    this.manager = manager;
  };

  /** Converts the template to string.
  * @return the string
  */
  public String toString () {
    StringBuffer sb = new StringBuffer (name);
    if (supr != null && supr != basic) {
      sb.append (':');
      sb.append (supr.name);
    };
    sb.append ('#');
    sb.append (manager.getClass ().getName ());

    String info = manager.storeInfo (this);
    if (info != null && !info.equals ("")) {
      sb.append (';');
      sb.append (info);
    };
    
    return sb.toString ();
  };

  /** Returns the name of the template.
  * @return the name
  */
  public String getName() {
    return name;
  }

  /** Creates new template from the string.
  * @param s the string
  * @return the new template
  * @exception AlreadyUsedException if the template name has already been registrated
  * @exception IllegalArgumentException if the string format is invalid
  * @exception ClassNotFoundException if the manager cannot be found
  * @exception IllegalAccessException if the instance of template manager
  *   cannot be created
  * @exception InstantiationException if the instance cannot be created
  * @exception ClassCastException if the manager class has been found but
  *   it is not descendant of TemplateManager interface
  * @exception NoSuchElementException if the super template has not been found
  * @see TemplateManager
  */
  public static Template fromString (String s) throws
  AlreadyUsedException,
  IllegalArgumentException,
  ClassNotFoundException,
  IllegalAccessException,
  InstantiationException,
  ClassCastException {
    StringTokenizer st = new StringTokenizer (s, ":#;", true);
    try {
      // name of template
      String name = st.nextToken ();
      String delim = st.nextToken ();
      Template supr = null;
      String clazzName;
      if (delim.equals (":")) {
        // read the super template
        supr = Registration.find (st.nextToken ());
        delim = st.nextToken ();
      } else {
        // if no super template is specified the basic one is used
        // as a parent
        supr = basic;
      };
      if (delim.equals ("#")) {
        // read the mangager name
        clazzName = st.nextToken ();
      } else {
        throw new IllegalArgumentException ();
      };

      // can throw ClassNotFoundException
      Class clazz = Class.forName (clazzName);
      TemplateManager manager;
      try {
        manager = (TemplateManager)clazz.newInstance ();
      } catch (NoSuchMethodError ex) {
        throw new InstantiationException ();
      };

      Template template = new TemplateImpl (name, supr, manager);

      String rest = "";
      try {
        if (st.nextToken ().equals (";")) {
          rest = st.nextToken ("");
        };
      } catch (NoSuchElementException ex) {
        // it is no error if the part after ; is missing
        // go on
      };
      
      manager.loadInfo (template, rest);

      template.treeNode = Registration.register (template);

      return template;
    } catch (NoSuchElementException ex) {
      // when there is not as much elements
      throw new IllegalArgumentException ();
    }
  }

  /** Returns the tree of all templates.
  * This tree can be set to XFTreeView and displayed.
  * @return the tree
  * @see XFTreeView
  */
  public static XFTree getTree () {
    return Registration.getTree ();
  };

  /** Reads templates from properties.
  * @param p the properties
  */
  public static void loadTemplates (EnhProperties p) {
    int count = p.getIntProperty ("xelfi.templates.count", -1);
    for (int i = 0; i < count; i++) {
      String s = p.getProperty ("xelfi.templates." + i);
      try {
        if (s != null) {
          Template t = fromString (s);
          // mark this template to be saved
          t.saveToProperties = true;
        };
      } catch (Exception ex) {
        System.err.println ("Cannot read template " + s + "\n" + ex);
        ex.printStackTrace ();
      };
    };
  };

  /** Save templates to the property list.
  * @param p the properties
  */
  public static void storeTemplates (EnhProperties p) {
    TemplateWriter tw = new TemplateWriter (p);
    // saves every node without the basic one
    getTree ().findNodeWithData (tw, null);
    p.setIntProperty ("xelfi.templates.count", tw.count);
  };

  /** Looks if there exists a template with name s in the system.
  * @param s the name of template
  * @return the template
  * @exception NoSuchElementException is thrown if the template is not found
  */
  public static Template find (String s) throws NoSuchElementException {
    return Registration.find (s);
  };

  /** Creates new source using this template.
  * @param xf the xelfi file for new source
  * @return the new source
  * @exception Exception if something goes wrong, the source is not empty or cannot
  *   be created, etc.
  */
  public Source newSource (XelfiFile xf) throws Exception {
    // register this source to the source loader
    // this prevents the process of identifiing the source
    TemplateLoader.table.put (xf.getPackageAndName (), this);

    try {
      Source s = new Source (xf);
      // create new source
      manager.newSource (this, s);

      return s;
    } catch (Exception ex) {
      // if the file is not empty and we cannot create it
      // we must deregister it
      TemplateLoader.table.remove (xf.getPackageAndName ());
      // rethrow exception
      throw ex;
    }
  }

  /** Creates new source using this template.
  * @param s the name of class
  * @return the new source
  * @exception Exception if something goes wrong
  */
  public Source newSource (String s) throws Exception {
    return newSource (XelfiFile.find (s));
  };

  /** Identifies source.
  * @param s the source
  * @return the template
  */
  public static Template identifySource (Source s) {
    return TemplateLoader.templateForSource (s);
  };

  /** Creates new template from source s.
  * @param n the name of template
  * @param s the source
  * @exception Exception if the new template cannot be created
  */
  public static Template newTemplate (String n, Source s) throws Exception {
    if (Registration.isRegistered (n)) {
      throw new AlreadyUsedException ("New template " + n + " cannot be created");
    };

    Template old = identifySource (s);
    Template t = new TemplateImpl (n, old, null);

    // ask the manager for creation of the template
    t.manager = old.manager.createTemplate (old, s, t);

    // if succeed than register new template
    t.treeNode = Registration.register (t);
    t.saveToProperties = true;

    return t;
  };
};










