/**
* Xelfi programming environment
*/

package xelfi.util;
import java.util.Enumeration;
import java.util.NoSuchElementException;
/**
* Binary self modifiable (splay) tree
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class BinaryTree extends Object {
  private BinaryTreeNode root;
  private BinaryTreeCompare comp;

  /**
  * creates new empty tree
  * this tree should redefine method compare
  * @see compare
  */
  public BinaryTree () {
    root = null;
    comp = null;
  }

  /**
  * creates new empty tree and specifies method to compare
  * @param p the objects that implements comparing of objects
  */
  public BinaryTree (BinaryTreeCompare p) {
    root = null;
    comp = p;
  }

  /**
  * compares two objects in this implementation by calling compare interface
  * specified in constructor (if specified) or can be redefined to do
  * something else
  *
  * @param o1 first object
  * @param o2 second object
  * @return the same constants like method compare in BinaryTreeCompare
  * @see BinaryTreeCompare
  */
  protected byte compare (BinaryTreeNode first, Object second) {
    try {
      return comp.compare (
        ((BinaryTreeNodeImpl)first).getObject (), second
      );
    } catch (ClassCastException e) {
      return BinaryTreeCompare.LESS;
    }
  }

  /**
  * finds object o or the "closest" one in tree and moves it to root
  * @param o object to find
  * @return LESS, EQUAL or GREATER
  */
  private byte findToRoot (Object o) {
    if (root == null) return BinaryTreeCompare.LESS;
    else {
      BinaryTreeNode p = root;
      byte Res;
      Loop: for (;;) {
        BinaryTreeNode n;

        Res = compare (p, o);
        switch (Res) {
          case BinaryTreeCompare.GREATER:
            n = p.getLeftSon ();
            break;
          case BinaryTreeCompare.LESS:
            n = p.getRightSon ();
            break;
          default:
            // equal
            break Loop;
        }
        if (n == null) break Loop;
        p = n;
      }
      // p is set to node that should be moved to root
      BinaryTreeOperation.toRoot (root = p);
      return Res;
    }
  }

  /** inserts new object to the tree
  * @param o the object to insert
  * @exception DuplicateException if such a object has already been inserted
  */
  public void insert (Object o) throws DuplicateException {
    BinaryTreeNode n = new BinaryTreeNodeImpl (o);
    BinaryTreeOperation.init (n);

    if (root == null) {
      root = n;
    } else {
      byte res = findToRoot (o);
      switch (res) {
        case BinaryTreeCompare.LESS:
          BinaryTreeOperation.insertRightSon (n, root);
          break;
        case BinaryTreeCompare.GREATER:
          BinaryTreeOperation.insertLeftSon (n, root);
          break;
        default:
          // equal
          throw new DuplicateException ();
      }
    }
  }

  /** deletes object from tree
  * @param o the object to delete
  * @exception java.util.NoSuchElementException if there is no such element
  */
  public void delete (Object o) throws java.util.NoSuchElementException {
    if (findToRoot (o) == BinaryTreeCompare.EQUAL) {
      // nalezeno
      root = BinaryTreeOperation.deleteNode (root);
    } else {
      throw new java.util.NoSuchElementException ();
    }
  }

  /** access object in tree
  * @param o the object to access
  * @return the object that is EQUAL with o and is in tree
  * @exception java.util.NoSuchElementException if there is no such element
  */
  public Object access (Object o) throws java.util.NoSuchElementException {
    if (findToRoot (o) == BinaryTreeCompare.EQUAL) {
      return ((BinaryTreeNodeImpl)root).getObject ();
    } else {
      throw new java.util.NoSuchElementException ();
    }
  }

  /** tests if this object is a member of tree
  * @param o the object
  * @return true/false
  */
  public boolean isMember (Object o) {
    try {
      access (o);
      return true;
    } catch (java.util.NoSuchElementException e) {
      return false;
    }
  }

  /** Returns the root of binary tree.
  * @return the root
  * @exception NoSuchElementException if the tree is empty
  */
  public Object getRoot () throws NoSuchElementException {
    if (root == null) throw new NoSuchElementException ();
    return ((BinaryTreeNodeImpl)root).getObject ();
  }

  /** gets enumeration of all objects in tree.
  * @warning if the tree is changed after call to this method, the enumeration
  *   is still changed
  * @return the enumeration of all objects in tree
  */
  public Enumeration elements () {
    return new BinaryTreeEnumeration (root);
  }

  /** converts to string */
  public String toString () {
    StringBuffer b = new StringBuffer ();
    BinaryTreeOperation.print (b, 0, root);
    return b.toString ();
  }
}

