/**
* Xelfi programming environment
*/

package xelfi.util;
/**
* This class contains static methods that operates on object that
* implement BinaryTreeNode interface.
* @see BinaryTreeNode
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class BinaryTreeOperation extends Object {
  /** text constant */
  private static final String ET_ROOTACCESS = "Access to parent of root node";
  private static final String ET_NONEXT = "No next node";

  /**
  * inaccessible constructor to forbid creation of instances
  */
  private BinaryTreeOperation () {
  }

  /**
  * initialize state of a node
  * @param n the node
  */
  public static void init (BinaryTreeNode n) {
    n.setParent (null);
    n.setLeftSon (null);
    n.setRightSon (null);
    n.setValue (1);
  }

  /**
  * make n be the left son of p. n must have no sons
  * @param n node (not null)
  * @param p parent node (not null)
  */
  public static synchronized void insertLeftSon (BinaryTreeNode n, BinaryTreeNode p) {
    // move the parent to root of its tree
    toRoot (p);

    n.setParent (p);
    n.setRightSon (null);
    BinaryTreeNode a = p.getLeftSon ();
    n.setLeftSon (a);
    int aValue;;
    if (a != null) {
      a.setParent (n);
      aValue = a.getValue ();
    } else {
      aValue = 0;
    }

    p.setLeftSon (n);

    n.setValue (aValue + 1);
    p.setValue (p.getValue () + 1);
  }

  /**
  * make n be the right son of p. n must have no sons
  * @param n node (not null)
  * @param p parent node (not null)
  */
  public static synchronized void insertRightSon (BinaryTreeNode n, BinaryTreeNode p) {
    // move the parent to root of its tree
    toRoot (p);

    n.setParent (p);
    n.setLeftSon (null);
    BinaryTreeNode a = p.getRightSon ();
    n.setRightSon (a);
    int aValue;;
    if (a != null) {
      a.setParent (n);
      aValue = a.getValue ();
    } else {
      aValue = 0;
    }

    p.setRightSon (n);

    n.setValue (aValue + 1);
    p.setValue (p.getValue () + 1);
  }


  /**
  * tests if node has no parent
  * @param n the node (must not be null otherwise an NullPointerException is
  *    thrown)
  * @return true if has no parent
  */
  public static boolean isRoot (BinaryTreeNode n) {
    return (n.getParent () == null);
  }

  /**
  * gets parent or throws exception if node is a root
  * @param n the node (not null)
  * @return parent node
  * @exception NodeAccessException if n is root
  */
  private static BinaryTreeNode getParent (BinaryTreeNode n)
  throws NodeAccessException {
    BinaryTreeNode a = n.getParent ();
    if (a == null) throw new NodeAccessException (ET_ROOTACCESS);
    return a;
  }

  /**
  * @warning can be used only withing the deleteNode method!!
  *
  * makes n root. Deletes links between n and its parent
  * @param n the node (not null)
  * @return parent of n
  * @exception NodeAccessException if n has already been made root
  */
  private static synchronized BinaryTreeNode makeRoot (BinaryTreeNode n)
  throws NodeAccessException {
    BinaryTreeNode p = getParent (n);
    // throws NodeAccessException if n is root
    if (isLeftSon (n)) {
      p.setLeftSon (null);
    } else {
      p.setRightSon (null);
    }
    n.setParent (null);


    // change the value in all nodes from p till the root of p's tree
/*    int a = n.getValue ();
    BinaryTreeNode g = p;
    do {
      g.setValue (g.getValue () - a);
      g = g.getParent ();
    } while (g != null);
*/
    return p;
  }

  /**
  * test if node is left child of parent
  * @param n the node (not null)
  * @return true if n is left child, false if not
  * @exception NodeAccessException if node n is root
  */
  public static boolean isLeftSon (BinaryTreeNode n)
  throws NodeAccessException {
    return getParent (n).getLeftSon () == n;
  }

  /**
  * test if node is right child of parent
  * @param n the node (not null)
  * @return true if n is right child, false if not
  * @exception NodeAccessException if node n is root
  */
  public static boolean isRightSon (BinaryTreeNode n)
  throws NodeAccessException {
    return !isLeftSon (n);
  }

  /**
  * rotate node up (changes its and its parent's positions preserving
  * in order state)
  * @param n the node
  * @exception NodeAccessException if n is root
  */
  public static synchronized void rotate (BinaryTreeNode n)
  throws NodeAccessException {
    BinaryTreeNode p = getParent (n);

    // set the link from grand parent
    BinaryTreeNode grand = p.getParent ();
    if (grand != null) {
      // if n has grandparent
      if (isRightSon (p)) {
        // if p is a right son
        grand.setRightSon (n);
        // then n becames new right son
      } else {
        grand.setLeftSon (n);
        // n becames new left son
      }
    }

    int nValue = n.getValue ();
    int pValue = p.getValue ();
    int y;
    int z;

    if (p.getRightSon () == n) {
      // we seem to be right son of p
      BinaryTreeNode nleft = n.getLeftSon ();

      n.setLeftSon (p);
      p.setRightSon (nleft);
      n.setParent (grand);
      p.setParent (n);
      if (nleft != null) nleft.setParent (p);

      // prepare values
      BinaryTreeNode nright = n.getRightSon ();
      y = nright == null ? 0 : nright.getValue ();
      BinaryTreeNode lpar = p.getLeftSon ();
      z = lpar == null ? 0 : lpar.getValue ();
    } else {
      BinaryTreeNode nright = n.getRightSon ();
      n.setRightSon (p);
      p.setLeftSon (nright);
      n.setParent (grand);
      p.setParent (n);
      if (nright != null) nright.setParent (p);

      // prepare values
      BinaryTreeNode nleft = n.getLeftSon ();
      y = nleft == null ? 0 : nleft.getValue ();
      BinaryTreeNode rpar = p.getRightSon ();
      z = rpar == null ? 0 : rpar.getValue ();
    }

    // changes values in the tree
    p.setValue (pValue - y - 1);
    n.setValue (nValue + z + 1);
  }

  /**
  * makes node n root of tree and preserves in order
  * @param n the node to rotate to root
  */
  public static void toRoot (BinaryTreeNode n) {
    try {
      while (!isRoot (n)) {
        BinaryTreeNode p = n.getParent ();
        if (isRoot (p)) {
          // if this is root then do single rotation
          rotate (n);
        } else {
          // both n and p has parent => no exception is thrown here
          boolean a = p.getLeftSon () == n;
          boolean b = isLeftSon (p);
          if (a == b) {
            // zig-zig
            rotate (p);
            rotate (n);
          } else {
            // zig-zag
            rotate (n);
            rotate (n);
          }
        }
      }
    } catch (NodeAccessException E) {
      // this exception is never thrown but we must catch it here to
      // follow java needs
    }
  }


  /**
  * tests if node n is in left subtree of node p
  * @param n the lower node in tree
  * @param p the upper node in tree
  * @result true if n is in left subtree of p. false if n is in right one.
  * @exception NodeAccessException if n is not in any subtree of p
  *   (also if n and p are equal)
  */
  public static boolean isInLeftSubTree (BinaryTreeNode n, BinaryTreeNode p)
  throws NodeAccessException {
    for (;;) {
      BinaryTreeNode a = getParent (n);
      if (a == p) break;
      n = a;
    }
    return isLeftSon (n);
  }

  /**
  * finds leftest node in subtree of n
  * @param n the node (not null)
  * @return leftest node
  */
  public static BinaryTreeNode leftestNode (BinaryTreeNode n) {
    for (;;) {
      BinaryTreeNode l = n.getLeftSon ();
      if (l == null) return n;
      n = l;
    }
  }

  /** look for next greater node in tree
  * @param n the node to search from
  * @return next node in inorder tree
  * @exception NodeAccessException if there is no next node
  */
  public static BinaryTreeNode nextNode (BinaryTreeNode n)
  throws NodeAccessException {
    toRoot (n);
    BinaryTreeNode p = n.getRightSon ();
    if (p == null) throw new NodeAccessException (ET_NONEXT);
    return leftestNode (p);
  }

  /**
  * deletes node from a subtree
  * @param n the node (not null)
  * @return new root of tree (null if tree becomes empty)
  */
  public static synchronized BinaryTreeNode deleteNode (BinaryTreeNode n) {
    try {
      toRoot (n);
      // makes n the root

      BinaryTreeNode r = n.getRightSon ();
      if (r == null) {
        // no right son, only say that left son is root (if exists)
        BinaryTreeNode l = n.getLeftSon ();
        if (l != null) {
          l.setParent (null);
          // l becames the root
        }
        return l;
      } else {
        // exist right son
        try {
          makeRoot (r);
          // r becomes root of its tree
        } catch (NodeAccessException e) {
          // never thrown
        }
        BinaryTreeNode l = leftestNode (r);
        toRoot (l);

        // l has no left son
        BinaryTreeNode ll = n.getLeftSon ();
        l.setLeftSon (ll);
        if (ll != null) {
          ll.setParent (l);
          l.setValue (l.getValue () + ll.getValue ());
        }
        
        return l;
      }
    } finally {
      init (n);
      // clears n
    }
  }

  /**
  * prints a node
  */

  public static void print (StringBuffer b, int x, BinaryTreeNode n) {
    if (n == null) {
      for (int i = 0; i < x; i++) b.append (' ');
      b.append ("null\n");
    } else {
      print (b, x + 2, n.getLeftSon ());
      for (int i = 0; i < x; i++) b.append (' ');
      b.append (n);
      b.append ('\n');
      print (b, x + 2, n.getRightSon ());
    }
  }

}





