/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.Label;
import java.awt.Event;
import java.io.File;
import xelfi.editor.*;
/**
* Class that displays message at the bottom of editor.
* The message can be changed to any text but it will disappear on
* insert, delete or cursor move.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class StatusLine extends Label implements
  SourceListener,
  DocumentListener,
  EditorListener,
  ActionPostListener
{

  /** the frame */
  JavaEditorFrame frame;

  /** text in editor */
  private String text = "";

  /** New status line.
  * @param f the frame
  */
  public StatusLine (JavaEditorFrame f) {
    super ("");
    Source s = (frame = f).getSource ();
    s.addSourceListener (this);
    Document d = s.getDocument ();
    d.addActionPostListener (this);
    d.addDocumentListener (this);
    frame.getTextEditor ().addEditorListener (this);
    setDef ();
  };

  /** Removes itself from the list of action controlers. */
  public void close () {
    Source s = frame.getSource ();
    s.removeSourceListener (this);
    Document d = s.getDocument ();
    d.removeActionPostListener (this);
    d.removeDocumentListener (this);
    frame.getTextEditor ().removeEditorListener (this);
  };

  /** Sets text to the status line. This text remains till a change
  * in editor is done.
  * @param s the text to set to the line
  */
  public void setText (String s) {
    super.setText ((frame.getSource ().isModified () ?  "Modified     " : "Synchronized ") + s);
    text = s;
  };

  /** Resets the text to default.
  */
  void setDef () {
    Source s = frame.getSource ();
    setText (s.getSource ().toString ());
  };

  /** Notifies that an insert has been done at position p.
  * @param n the begin of inserted text
  * @param p the end of inserted text
  */
  public void notifyInsert (Position n, Position p) {
    setDef ();
  };

  /** Notifies a delete
  * @param p the position where a delete occures
  * @param s the string with characters that has been deleted
  */
  public void notifyDelete (Position p, String s) {
    setDef ();
  };

  /** This is called when the modified flag of the editor changes.
  * Either turns to modified or unmodified.
  * @param d document
  * @param mod true if the document is now modified
  */
  public void documentModified (Document d, boolean mod) {
    setText (text);
  };

  /** When someone sends a message to document this method is called.
  * @param d document
  * @param s string
  */
  public void documentMessage (Document d, String s) {
    setText (s);
  };

  /** This is called when the source is saved.
  * @param s the source
  */
  public void sourceSaved (Source s) {
    setText ("The document has been successfully saved");
  };

  /** Called when action requested by user in editor is not allowed.
  * @param info info about document
  * @param ins true if insert, false if delete
  */
  public void editorNotAllowedAction (TextEditorInfo info) {
    setText ("Action is not allowed at current position");
  };

  /** When user double clicks with mouse this will be called.
  * @param info editor info
  * @param p position of click
  */
  public void editorDoubleClick (TextEditorInfo info, Position p) {
  };

  /** Notifies change of view (visible part of document)
  * @param info editor info
  */
  public void editorViewChanged (TextEditorInfo info) {
  };
};










